#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   signal 
@Time        :   2023/11/21 14:06
@Author      :   Xuesong Chen
@Description :   
"""
import matplotlib.pyplot as plt
import numpy as np
from numpy.typing import NDArray
from hrvanalysis import get_nn_intervals
import neurokit2 as nk
from scipy.signal import filtfilt
import typing as t

def interp_heartbeat(positions, values, fs, target_fs, n_samples):
    '''
    Interpolate RRI or JJI to target sampling frequency
    Parameters
    ----------
    positions: positions of RRI or JJI
    values: values of RRI or JJI
    fs: sampling frequency of RRI or JJI
    target_fs: target sampling frequency
    n_samples: number of samples of the raw signal

    Returns
    -------
    interp_value: interpolated RRI or JJI at target sampling frequency, in ms
    '''
    rri_in_ms = values / fs * 1000
    filtered_rri_in_ms = get_nn_intervals(rri_in_ms, verbose=False, ectopic_beats_removal_method='karlsson')
    interp_value = np.interp(
        np.arange(0, n_samples, fs / target_fs), positions,
        filtered_rri_in_ms).astype(np.float32)
    if np.isnan(interp_value).all():
        raise ValueError('All values are nan')
    interp_value[np.isnan(interp_value)] = np.nanmean(interp_value)
    return interp_value


def split_array_by_nan(arr):
    """
    Splits an array into multiple sub-arrays where each sub-array either contains
    only NaNs or contains no NaNs at all.

    Parameters:
    arr (np.ndarray): The input array to be split.

    Returns:
    list of np.ndarray: A list of sub-arrays split according to NaN values.
    """
    # 找到NaN的索引
    nan_indices = np.where(np.isnan(arr))[0]

    # 如果数组中没有NaN，返回原数组
    if nan_indices.size == 0:
        return [arr]

    # 添加分割点
    split_indices = []
    if nan_indices[0] != 0:
        split_indices.append(0)
    for i in range(len(nan_indices) - 1):
        if nan_indices[i] + 1 != nan_indices[i + 1]:
            split_indices.extend([nan_indices[i] + 1, nan_indices[i + 1]])
    if nan_indices[-1] != len(arr) - 1:
        split_indices.append(nan_indices[-1] + 1)

    # 分割数组
    result = np.split(arr, split_indices)

    # 过滤结果，确保每个部分要么全是NaN，要么全不是NaN
    filtered_result = []
    for subarray in result:
        if np.isnan(subarray).all() or not np.isnan(subarray).any():
            filtered_result.append(subarray)

    return filtered_result


def filter_spo2(spo2, fs, min_allowed_spo2=50, max_allowed_spo2=105):
    """
    This function filters the SpO2 signal based on certain conditions.

    Parameters:
    spo2 (numpy array): The SpO2 signal data.
    fs (int): The sampling frequency of the SpO2 signal.
    min_allowed_spo2 (int, optional): The minimum allowed SpO2 value. Defaults to 50.
    max_allowed_spo2 (int, optional): The maximum allowed SpO2 value. Defaults to 105.

    Returns:
    spo2 (numpy array): The filtered SpO2 signal data.
    """
    def bandpass_filter(spo2):
        B = [0.000109398212241, 0.000514594526374, 0.001350397179936, 0.002341700062534,
             0.002485940327008, 0.000207543145171, -0.005659450344228, -0.014258087808069,
             -0.021415481383353, -0.019969417749860, -0.002425120103463, 0.034794452821365,
             0.087695691366900, 0.144171828095816, 0.187717212244959, 0.204101948813338,
             0.187717212244959, 0.144171828095816, 0.087695691366900, 0.034794452821365,
             -0.002425120103463, -0.019969417749860, -0.021415481383353, -0.014258087808069,
             -0.005659450344228, 0.000207543145171, 0.002485940327008, 0.002341700062534,
             0.001350397179936, 0.000514594526374, 0.000109398212241]
        if len(spo2) <= len(B) * 3:
            return spo2
        else:
            return filtfilt(B, 1, spo2, axis=0, padtype='odd')

    if fs != 1:
        spo2 = nk.signal_resample(spo2, sampling_rate=fs, desired_sampling_rate=1)
    # 如果下降速率大于5，或者上升速率大于10，则认为是异常值
    max_drop_rate = -5
    max_recover_rate = 10
    mask = np.logical_or(np.diff(spo2) < max_drop_rate, np.diff(spo2) > max_recover_rate)
    spo2[1:][mask] = np.nan
    mask = np.logical_or(spo2 < min_allowed_spo2, spo2 > max_allowed_spo2)
    spo2[mask] = np.nan
    return spo2
    # spo2_arr = split_array_by_nan(spo2)
    # ret_spo2 = np.array([])
    # for _arr in spo2_arr:
    #     if np.isnan(_arr).all():
    #         ret_spo2 = np.concatenate([ret_spo2, _arr])
    #     else:
    #         ret_spo2 = np.concatenate([ret_spo2, bandpass_filter(_arr)])
    # return ret_spo2


def get_eeg_features(
        eeg: NDArray,
        fs: t.Union[int, float],
        seg_duration:  t.Union[int, float] = 1,
        seg_overlap_ratio: t.Union[int, float] = 0.5,
) -> NDArray:
    """
    This function calculates the normalized power of an EEG signal.

    Parameters:
    eeg (numpy array): The EEG signal data.
    fs (int): The sampling frequency of the EEG signal.
    seg_duration (int, optional): The duration of each segment in seconds. Defaults to 1.
    seg_overlap_ratio (float, optional): The overlap ratio of each segment. Defaults to 0.5.

    Returns:
    norm_power (numpy array): The normalized power of the EEG signal. The dimensions are frequency by time.
    """
    freq, time, power = nk.signal_timefrequency(
        eeg,
        window=seg_duration,
        overlap=fs * seg_duration * seg_overlap_ratio,
        min_frequency=0.3,
        max_frequency=35,
        sampling_rate=fs,
        show=False)
    norm_power = power / (np.sum(power, axis=0, keepdims=True) + 1e-8)
    return norm_power  # freq * time



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   Classifier
@Time        :   2023/9/21 21:47
@Author      :   Xuesong Chen
@Description :   
"""

from sklearn.metrics import (accuracy_score, precision_score, recall_score,
                             f1_score, roc_auc_score, confusion_matrix, cohen_kappa_score)


class ClassificationEvaluator:
    def __init__(self, true_labels, predicted_labels, predicted_probabilities=None):
        self.true_labels = true_labels
        self.predicted_labels = predicted_labels
        self.predicted_probabilities = predicted_probabilities

    def accuracy(self):
        return accuracy_score(self.true_labels, self.predicted_labels)

    def precision(self, average='macro'):
        return precision_score(self.true_labels, self.predicted_labels, average=average)

    def recall(self, average='macro'):
        return recall_score(self.true_labels, self.predicted_labels, average=average)

    def f1(self, average='macro'):
        return f1_score(self.true_labels, self.predicted_labels, average=average)

    def auc(self, average='macro'):
        # 需要确保预测概率被传入
        if self.predicted_probabilities is None:
            raise ValueError("Predicted probabilities are required for AUC computation.")
        # AUC计算只在二分类或标签指示格式的多标签分类中有意义
        return roc_auc_score(self.true_labels, self.predicted_probabilities, multi_class='ovr', average=average)

    def kappa(self):
        return cohen_kappa_score(self.true_labels, self.predicted_labels)

    def confusion_matrix(self, normalize='true', labels=None):
        return confusion_matrix(self.true_labels, self.predicted_labels, normalize=normalize, labels=labels)



import ctypes
import numpy as np
import pandas as pd
import os

# Load the shared library
current_dir = os.path.dirname(os.path.abspath(__file__))
lib_path = os.path.join(current_dir, 'desaturation.so')
lib = ctypes.CDLL(lib_path)


# Define the return structure from the C function
class DesaturationResult(ctypes.Structure):
    _fields_ = [
        ("onset", ctypes.POINTER(ctypes.c_int)),
        ("duration", ctypes.POINTER(ctypes.c_int)),
        ("level", ctypes.POINTER(ctypes.c_float)),  # Adjusted for float
        ("size", ctypes.c_int)
    ]


# Specify argument and return types
lib.detect_oxygen_desaturation.argtypes = [
    ctypes.POINTER(ctypes.c_float),  # spo2_arr now expects floats
    ctypes.c_int,  # arr_size
    ctypes.c_int,  # duration_max
    ctypes.c_float  # spo2_des_min_thre also as float
]
lib.detect_oxygen_desaturation.restype = DesaturationResult
lib.free_memory.argtypes = [ctypes.c_void_p]


def detect_oxygen_desaturation(spo2_arr, duration_max=120, spo2_des_min_thre=3):
    spo2_data = spo2_arr.astype(np.float32)  # Ensure the data is in float32 format
    spo2_data_ptr = spo2_data.ctypes.data_as(ctypes.POINTER(ctypes.c_float))  # Pointer to float
    arr_size = len(spo2_data)
    # Call the C function
    result = lib.detect_oxygen_desaturation(spo2_data_ptr, arr_size, duration_max, spo2_des_min_thre)

    # Convert results from C to Python
    onset_array = np.ctypeslib.as_array(result.onset, (result.size,)).copy()
    duration_array = np.ctypeslib.as_array(result.duration, (result.size,)).copy()
    level_array = np.ctypeslib.as_array(result.level, (result.size,)).copy()  # Correctly handle as float

    # Free allocated memory in C
    lib.free_memory(result.onset)
    lib.free_memory(result.duration)
    lib.free_memory(result.level)

    return pd.DataFrame({
        'Type': 'OD',
        'Start': onset_array,
        'Duration': duration_array,
        'OD_level': level_array
    })




#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   __init__.py 
@Time        :   2023/8/17 16:45
@Author      :   Xuesong Chen
@Description :   
"""

from .Annotation.NSRR import NSRRAnnotationReader
from .Annotation.Philips import PhilipsAnnotationReader
from .Annotation.HSP import HSPAnnotationReader, HSPHumanInfoReader
from .Annotation.Base import AHI

from .EDF.NSRR import NSRREDFReader
from .EDF.Philips import PhilipsEDFReader
from .EDF.HSP import HSPEDFReader as HSPMATReader
from .EDF.HSPV2 import HSPV2EDFReader as HSPEDFReader




import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

def plot_signal_with_events(signal, fs, events_df):
    """
    绘制信号和事件的时序图。

    参数：
    - signal: ndarray，信号数据
    - fs: float，采样率（Hz）
    - events_df: DataFrame，包含事件信息的DataFrame，必须包含以下列：'Type', 'Start', 'Duration'
    """

    # 生成时间轴
    duration = len(signal) / fs
    t = np.linspace(0, duration, len(signal), endpoint=False)

    # 绘制信号
    plt.figure(figsize=(10, 6))
    plt.plot(t, signal, label='Signal')

    # 用于存储事件类型的标签
    event_labels = {}

    # 绘制事件
    for idx, row in events_df.iterrows():
        event_type = row['Type']
        if event_type not in event_labels:
            start_time = row['Start']
            duration = row['Duration']
            plt.axvspan(start_time, start_time + duration, color='red', alpha=0.5, label=event_type)
            event_labels[event_type] = True
        else:
            start_time = row['Start']
            duration = row['Duration']
            plt.axvspan(start_time, start_time + duration, color='red', alpha=0.5)

    # 设置图例
    plt.legend()

    # 设置标题和标签
    plt.title('Signal with Events')
    plt.xlabel('Time (s)')
    plt.ylabel('Amplitude')

    # 显示图形
    plt.grid(True)
    plt.show()

if __name__ == '__main__':
    # 示例信号
    fs = 1000  # 采样率（Hz）
    duration = 10  # 信号的总持续时间（秒）
    t = np.linspace(0, duration, duration * fs, endpoint=False)
    signal = np.random.randn(len(t))

    # 示例事件的DataFrame
    events_df = pd.DataFrame({
        'Type': ['Event1', 'Event2', 'Event1'],
        'Start': [2, 6, 4],
        'Duration': [1, 1.5, 0.5]
    })

    # 绘制信号和事件
    plot_signal_with_events(signal, fs, events_df)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Metrics
-------

Main functions:

* :func:`sed_eval.scene.SceneClassificationMetrics.evaluate`: Calculate intermediate values for evaluation and accumulate them.
* :func:`sed_eval.scene.SceneClassificationMetrics.results`: Calculate and return all metrics.
* :func:`sed_eval.scene.SceneClassificationMetrics.results_overall_metrics`: Calculate and return overall metrics (micro-averaged).
* :func:`sed_eval.scene.SceneClassificationMetrics.results_class_wise_metrics`: Calculate and return class-wise metrics.
* :func:`sed_eval.scene.SceneClassificationMetrics.results_class_wise_average_metrics`: Calculate and return class-wise average metrics (macro-averaged).

Function :func:`sed_eval.scene.SceneClassificationMetrics.evaluate` takes as a parameter scene lists,
use :func:`sed_eval.io.load_scene_list` to read them from a file.

Usage example to evaluate files:

.. code-block:: python
    :linenos:
    import sed_eval
    import dcase_util

    file_list = [
        {'reference_file': 'fold1_reference.txt', 'estimated_file': 'fold1_estimated.txt'}
    ]

    data = []

    # Get used scene labels and load data in
    all_data = []
    for file_pair in file_list:
        reference_scene_list = sed_eval.io.load_scene_list(
            filename=file_pair['reference_file'],
            csv_header=False,
            file_format=dcase_util.utils.FileFormat.CSV,
            fields=['filename', 'scene_label']
        )
        estimated_scene_list = sed_eval.io.load_scene_list(
            filename=file_pair['estimated_file'],
            csv_header=False,
            file_format=dcase_util.utils.FileFormat.CSV,
            fields=['filename', 'onset', 'offset', 'scene_label']
        )

        data.append(
            {
                'reference_scene_list': reference_scene_list,
                'estimated_scene_list': estimated_scene_list
            }
        )

        all_data += reference_scene_list

    scene_labels = sed_eval.sound_event.util.unique_scene_labels(all_data)

    # Create metrics class
    scene_metrics = sed_eval.scene.SceneClassificationMetrics(
        scene_labels=scene_labels
    )
    for file_pair in data:
        scene_metrics.evaluate(
            reference_scene_list=file_pair['reference_scene_list'],
            estimated_scene_list=file_pair['estimated_scene_list']
        )

    # Get only certain metrics
    overall_metrics_results = scene_metrics.results_overall_metrics()
    print("Accuracy:", overall_metrics_results['accuracy'])

    # Or print all metrics as reports
    print(scene_metrics)

Usage example to evaluate results stored in variables:

.. code-block:: python
    :linenos:

    import sed_eval
    import dcase_util

    reference = dcase_util.containers.MetaDataContainer([
        {
            'scene_label': 'supermarket',
            'file': 'supermarket09.wav'
        },
        {
            'scene_label': 'tubestation',
            'file': 'tubestation10.wav'
        },
        {
            'scene_label': 'quietstreet',
            'file': 'quietstreet08.wav'
        },
        {
            'scene_label': 'office',
            'file': 'office10.wav'
        },
        {
            'scene_label': 'bus',
            'file': 'bus01.wav'
        },
    ])

    estimated = dcase_util.containers.MetaDataContainer([
        {
            'scene_label': 'supermarket',
            'file': 'supermarket09.wav'
        },
        {
            'scene_label': 'bus',
            'file': 'tubestation10.wav'
        },
        {
            'scene_label': 'quietstreet',
            'file': 'quietstreet08.wav'
        },
        {
            'scene_label': 'park',
            'file': 'office10.wav'
        },
        {
            'scene_label': 'car',
            'file': 'bus01.wav'
        },
    ])

    scene_labels = sed_eval.sound_event.util.unique_scene_labels(reference)

    scene_metrics = sed_eval.scene.SceneClassificationMetrics(scene_labels)
    scene_metrics.evaluate(
        reference_scene_list=reference,
        estimated_scene_list=estimated
    )

    print(scene_metrics)

.. autosummary::
    :toctree: generated/

    SceneClassificationMetrics
    SceneClassificationMetrics.evaluate
    SceneClassificationMetrics.results
    SceneClassificationMetrics.results_overall_metrics
    SceneClassificationMetrics.results_class_wise_metrics
    SceneClassificationMetrics.results_class_wise_average_metrics
    SceneClassificationMetrics.result_report_parameters
    SceneClassificationMetrics.result_report_class_wise
    SceneClassificationMetrics.result_report_class_wise_average
    SceneClassificationMetrics.reset

"""

from __future__ import absolute_import
import numpy
import dcase_util
from . import metric


class SceneClassificationMetrics:
    def __init__(self, scene_labels=None):
        self.accuracies_per_class = None
        self.scene_label_list = scene_labels

        self.overall = {
            'Ncorr': 0.0,
            'Nref': 0.0,
            'Nsys': 0.0
        }

        self.scene_wise = {}
        for label in self.scene_label_list:
            self.scene_wise[label] = {
                'Ncorr': 0.0,
                'Nref': 0.0,
                'Nsys': 0.0
            }

        self.ui = dcase_util.ui.FancyStringifier()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        return self.results()

    def __str__(self):
        """Print result reports"""

        output = self.ui.section_header('Scene classification metrics') + '\n'
        output += self.result_report_parameters() + '\n'
        output += self.result_report_class_wise_average() + '\n'
        output += self.result_report_class_wise() + '\n'

        return output

    def evaluate(self, reference_scene_list, estimated_scene_list=None, estimated_scene_probabilities=None):
        """Evaluate file pair (reference and estimated)

        Parameters
        ----------

        reference_scene_list : list of dict or dcase_util.containers.MetaDataContainer
            Reference scene list.
            Default value None

        estimated_scene_list : list of dict or dcase_util.containers.MetaDataContainer
            Estimated scene list.
            Default value None

        estimated_scene_probabilities : dcase_util.containers.ProbabilityContainer
            Estimated scene probabilities. Currently not used.
            Default value None

        Returns
        -------
        self

        """

        if estimated_scene_list is None and estimated_scene_probabilities is None:
            raise ValueError("Nothing to evaluate, give at least estimated_scene_list or estimated_scene_probabilities")

        # Make sure reference_scene_list is dcase_util.containers.MetaDataContainer
        if not isinstance(estimated_scene_list, dcase_util.containers.MetaDataContainer):
            reference_scene_list = dcase_util.containers.MetaDataContainer(reference_scene_list)

        # Make sure estimated_scene_list is dcase_util.containers.MetaDataContainer
        if not isinstance(estimated_scene_list, dcase_util.containers.MetaDataContainer):
            estimated_scene_list = dcase_util.containers.MetaDataContainer(estimated_scene_list)

        # Make sure estimated_tag_probabilities is dcase_util.containers.ProbabilityContainer
        if estimated_scene_probabilities is not None:
            if not isinstance(estimated_scene_probabilities, dcase_util.containers.ProbabilityContainer):
                estimated_scene_probabilities = dcase_util.containers.ProbabilityContainer(estimated_scene_probabilities)

        # Translate "file" field to "filename"
        for item in reference_scene_list:
            if 'filename' not in item and 'file' in item:
                item['filename'] = item['file']

        for item in estimated_scene_list:
            if 'filename' not in item and 'file' in item:
                item['filename'] = item['file']

        y_true = []
        y_pred = []

        for estimated_item in estimated_scene_list:
            reference_item_matched = {}
            for reference_item in reference_scene_list:
                if estimated_item['filename'] == reference_item['filename']:
                    reference_item_matched = reference_item
                    break

            if not reference_item_matched:
                raise ValueError(
                    "Cannot find reference_item for estimated item [{item}]".format(item=estimated_item['file'])
                )

            y_true.append(reference_item_matched['scene_label'])
            y_pred.append(estimated_item['scene_label'])

        y_true = numpy.array(y_true)
        y_pred = numpy.array(y_pred)

        Ncorr_overall = 0
        for scene_id, scene_label in enumerate(self.scene_label_list):
            true_id = numpy.where(y_true == scene_label)[0]
            pred_id = numpy.where(y_pred == scene_label)[0]

            Ncorr = 0
            for id in true_id:
                if id in pred_id:
                    Ncorr += 1

            Ncorr_overall += Ncorr
            self.scene_wise[scene_label]['Ncorr'] += Ncorr
            self.scene_wise[scene_label]['Nref'] += true_id.shape[0]
            self.scene_wise[scene_label]['Nsys'] += pred_id.shape[0]

        self.overall['Ncorr'] += Ncorr_overall
        self.overall['Nref'] += y_true.shape[0]
        self.overall['Nsys'] += y_pred.shape[0]

        return self

    def reset(self):
        """Reset internal state
        """
        self.overall = {
            'Ncorr': 0.0,
            'Nref': 0.0,
            'Nsys': 0.0
        }

        self.scene_wise = {}
        for label in self.scene_label_list:
            self.scene_wise[label] = {
                'Ncorr': 0.0,
                'Nref': 0.0,
                'Nsys': 0.0
            }

    # Reports
    def result_report_parameters(self):
        """Report metric parameters

        Returns
        -------
        str
            result report in string format

        """

        output = self.ui.data(field='Scene labels', value=len(self.scene_label_list)) + '\n'
        output += self.ui.data(field='Evaluated units', value=int(self.overall['Nref'])) + '\n'

        return output

    def result_report_class_wise(self):
        """Report class-wise results

        Returns
        -------
        str
            result report in string format

        """

        results = self.results_class_wise_metrics()

        output = self.ui.section_header('Class-wise metrics', indent=2) + '\n'

        output += self.ui.row(
            'Scene label', 'Ncorr', 'Nref', 'Accuracy',
            widths=[20, 12, 12, 12],
            separators=[True, False, True, False],
            indent=4
        ) + '\n'

        output += self.ui.row('-', '-', '-', '-') + '\n'

        for scene_label in self.scene_label_list:
            output += self.ui.row(
                scene_label,
                results[scene_label]['count']['Ncorr'],
                results[scene_label]['count']['Nref'],
                results[scene_label]['accuracy']['accuracy'] * 100,
                types=['str', 'int', 'int', 'float1_percentage']
            ) + '\n'

        return output

    def result_report_class_wise_average(self):
        """Report class-wise averages

        Returns
        -------
        str
            result report in string format

        """

        results = self.results_class_wise_average_metrics()

        output = self.ui.section_header('Class-wise average metrics (macro-average)', indent=2) + '\n'

        output += self.ui.line('Accuracy', indent=2) + '\n'
        output += self.ui.data(field='Accuracy',
                               value=float(results['accuracy']['accuracy']) * 100, unit='%', indent=4) + '\n'

        return output

    # Results
    def results(self):
        """All metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        results = {
            'overall': self.results_overall_metrics(),
            'class_wise': self.results_class_wise_metrics(),
            'class_wise_average': self.results_class_wise_average_metrics()
        }

        return results

    def results_overall_metrics(self):
        """Overall metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        if self.overall['Nsys'] != 0:
            accuracy = self.overall['Ncorr'] / float(self.overall['Nsys'])
        else:
            accuracy = None

        return {
            'count': self.overall,
            'accuracy': accuracy
        }

    def results_class_wise_metrics(self):
        """Class-wise metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        results = {}
        for scene_id, scene_label in enumerate(self.scene_label_list):
            if scene_label not in results:
                results[scene_label] = {}

            results[scene_label]['count'] = {}
            results[scene_label]['count']['Ncorr'] = self.scene_wise[scene_label]['Ncorr']
            results[scene_label]['count']['Nref'] = self.scene_wise[scene_label]['Nref']
            results[scene_label]['count']['Nsys'] = self.scene_wise[scene_label]['Nsys']
            results[scene_label]['accuracy'] = {
                'accuracy': metric.accuracy_corr(
                    Ncorr=self.scene_wise[scene_label]['Ncorr'],
                    N=self.scene_wise[scene_label]['Nref']
                )
            }

        return results

    def results_class_wise_average_metrics(self):
        """Class-wise averaged metrics

        Returns
        -------
        dict
            results in a dictionary format

        """

        scene_wise_results = self.results_class_wise_metrics()

        scene_wise_accuracy = []
        for scene_label in scene_wise_results:
            scene_wise_accuracy.append(scene_wise_results[scene_label]['accuracy']['accuracy'])

        return {
            'accuracy': {
                'accuracy': float(numpy.mean(scene_wise_accuracy))
            }
        }

    # Metrics
    def class_wise_accuracy(self, scene_label):
        """Class-wise accuracy

        Returns
        -------
        dict
            results in a dictionary format

        """

        if len(self.accuracies_per_class.shape) == 2:
            return {
                'accuracy': float(numpy.mean(self.accuracies_per_class[:, self.scene_label_list.index(scene_label)]))
            }

        else:
            return {
                'accuracy': float(numpy.mean(self.accuracies_per_class[self.scene_label_list.index(scene_label)]))
            }



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   sort 
@Time        :   2024/7/19 14:41
@Author      :   Xuesong Chen
@Description :   
"""

def create_sort_key(preferences):
    def sort_key(item):
        item = str(item)
        for index, group in enumerate(preferences):
            if any(pref in item for pref in group):
                return index
        return len(preferences)  # 不在偏好列表中的项排在最后
    return sort_key


if __name__ == '__main__':
    preferences = [
        ('C3', 'C4'),
        ('F3', 'F4'),
    ]
    sort_key = create_sort_key(preferences)
    items = ['F3-M1','C3-M1', 'O1-M1', 'C4-M2', 'F4-M2', 'O2-M2']
    items = sorted(items, key=None)
    print(items)
